package nodeinfo

import (
	"fmt"
	"net/http"
)

type Extension interface {
	Name() string
	Extend(client *http.Client, domain string, node *Node) error
}

var (
	extensions           = map[string]Extension{}
	genericFix Extension = nil
)

func RegisterExtension(e Extension) {
	if e != nil {
		extensions[e.Name()] = e
	}
}

func RegisterGenericExtension(e Extension) {
	if e != nil {
		genericFix = e
	}
}

func Extend(client *http.Client, domain string, node *Node) error {
	nativeVersion := node.Version

	if extender, extend := extensions[node.Software.Name]; extend {
		if err := extender.Extend(client, domain, node); err != nil {
			return fmt.Errorf("software specific extender [%s] failed: %w", extender.Name(), err)
		}
	}

	if genericFix != nil {
		if err := genericFix.Extend(client, domain, node); err != nil {
			return fmt.Errorf("generic extender failed: %w", err)
		}
	}

	// if if nodeinfo got altered save original one
	if _, exist := node.Metadata["nativeNodeinfoVersion"]; !exist && nativeVersion != node.Version {
		node.Metadata["nativeNodeinfoVersion"] = fmt.Sprintf(`"nodeinfo": "%s"`, nativeVersion)
	}

	return nil
}
